@@ -1,10 +1,9 @@
 from .. import scpi
 import sys
 import numpy as np
-import pyvisa as visa
 
 
-def _read_fixed_bytes(inst: scpi.Instrument | visa.resources.Resource, size: int) -> bytes:
+def _read_fixed_bytes(inst, size: int) -> bytes:
     data = inst.read_bytes()
     while len(data) < size:
         data += inst.read_bytes()
@@ -12,16 +11,18 @@ def _read_fixed_bytes(inst: scpi.Instrument | visa.resources.Resource, size: int
     return data
 
 
-def _readWaveDate(inst: scpi.Instrument | visa.resources.Resource, channel: int, points: int) -> np.ndarray:
+def _readWaveDate(inst, channel: int, points: int, debug: bool = False) -> np.ndarray:
 
     inst.query('*OPC?')
     inst.write(f':WAVeform:SOURce CHANnel{channel}')
     inst.query('*OPC?')
-    print('Reading channel ' + str(channel))
+    _log('Reading channel ' + str(channel), debug)
 
     inst.write(':WAVeform:FORMat BYTE')
     inst.write(':WAVeform:POINts:MODE MAXimum')
 
+    _log('Reading points', debug)
+
     if points > 0:
         inst.write(f':WAVeform:POINts {points}')
     else:
@@ -29,9 +30,11 @@ def _readWaveDate(inst: scpi.Instrument | visa.resources.Resource, channel: int,
 
     inst.query('*OPC?')
 
+    _log('Reading preamble', debug)
+
     peram = inst.query(':WAVeform:PREamble?')
     peram = peram.split(',')
-    print(peram)
+    _log(peram, debug)
 
     format = peram[0]
     type = peram[1]
@@ -43,14 +46,16 @@ def _readWaveDate(inst: scpi.Instrument | visa.resources.Resource, channel: int,
     yorg = float(peram[8])
     yref = float(peram[9])
 
+    _log('Reading data', debug)
+
     inst.write(':WAVeform:DATA?')
     data = _read_fixed_bytes(inst, int(ppoints))
 
     header = data[2:10].decode('utf-8')
-    print(header)
+    _log(header, debug)
     rpoints = int(header)
 
-    if rpoints != (points):
+    if rpoints != (ppoints):
         print('ERROR: points mismatch, please investigate')
 
     data = data[10:-1]
@@ -62,30 +67,31 @@ def _readWaveDate(inst: scpi.Instrument | visa.resources.Resource, channel: int,
     return voltCH
 
 
-def readChannels(inst: scpi.Instrument | visa.resources.Resource, channels: list[int], points: int = 0, runAfter: bool = True) -> tuple[np.ndarray, np.ndarray]:
+def readChannels(inst, channels: list[int], points: int = 0, runAfter: bool = True, debug: bool = False) -> tuple[np.ndarray, np.ndarray]:
     """Reads multiple channels from the oscilloscope.
 
-    :param inst: The instrument object
+    :param inst: The instrument object from pyscpi or pyvisa
     :param channels: A list of channels to read eg. [1, 2]
     :param points: The number of points to read. If 0, read all points
     :param runAfter: Run the oscilloscope after reading
+    :param debug: Print debug messages
     :return: A NumPy tuple of time and voltage arrays
 
     """
 
     inst.write(':TIMebase:MODE MAIN')
 
-    channelCommand = 'CHANnel1, CHANnel2'
-    # for channel in channels:
-    #    channelCommand = channelCommand + f'CHANnel{channel}, '
+    channelCommand = ''
+    for channel in channels:
+        channelCommand = channelCommand + f'CHANnel{channel}, '
 
-    print(channelCommand)
+    _log(channelCommand, debug)
 
     inst.write(f':DIGitize {channelCommand}')
 
     peram = inst.query(':WAVeform:PREamble?')
     peram = peram.split(',')
-    print(peram)
+    _log(peram, debug)
 
     format = peram[0]
     type = peram[1]
@@ -97,7 +103,7 @@ def readChannels(inst: scpi.Instrument | visa.resources.Resource, channels: list
     allData = np.empty([ppoints, len(channels)])
 
     for i in range(len(channels)):
-        print(f'Reading channel {channels[i]}')
+        _log(f'Reading channel {channels[i]}', debug)
         data = _readWaveDate(inst, channels[i], points)
         allData[:, i] = data
 
@@ -109,13 +115,14 @@ def readChannels(inst: scpi.Instrument | visa.resources.Resource, channels: list
     return time, allData
 
 
-def readSingleChannel(inst: scpi.Instrument | visa.resources.Resource, channel: int, points: int = 0, runAfter: bool = True) -> tuple[np.ndarray, np.ndarray]:
+def readSingleChannel(inst, channel: int, points: int = 0, runAfter: bool = True, debug: bool = False) -> tuple[np.ndarray, np.ndarray]:
     """Reads a single channel from the oscilloscope.
 
-    :param inst: The instrument object
+    :param inst: The instrument object from pyscpi or pyvisa
     :param channel: The channel to read
     :param points: The number of points to read. If 0, read all points
     :param runAfter: Run the oscilloscope after reading
+    :param debug: Print debug messages
     :return: A NumPy tuple of time and voltage arrays
 
     """
@@ -136,7 +143,7 @@ def readSingleChannel(inst: scpi.Instrument | visa.resources.Resource, channel:
 
     peram = inst.query(':WAVeform:PREamble?')
     peram = peram.split(',')
-    print(peram)
+    _log(peram, debug)
 
     format = peram[0]
     type = peram[1]
@@ -148,17 +155,19 @@ def readSingleChannel(inst: scpi.Instrument | visa.resources.Resource, channel:
     yorg = float(peram[8])
     yref = float(peram[9])
 
+    _log('Reading data', debug)
+
     inst.write(':WAVeform:DATA?')
     data = _read_fixed_bytes(inst, int(ppoints))
 
-    header = data[2: 10].decode('utf-8')
-    print(header)
+    header = data[2:10].decode('utf-8')
+    _log(header, debug)
     rpoints = int(header)
 
     if rpoints != (ppoints):
         print('ERROR: points mismatch, please investigate')
 
-    data = data[10: -1]
+    data = data[10:-1]
 
     data = np.frombuffer(data, dtype=np.uint8)
 
@@ -172,20 +181,20 @@ def readSingleChannel(inst: scpi.Instrument | visa.resources.Resource, channel:
     return time, voltCH
 
 
-def autoScale(inst: scpi.Instrument | visa.resources.Resource) -> None:
+def autoScale(inst) -> None:
     """Autoscales the oscilloscope.
 
-    :param inst: The instrument object
+    :param inst: The instrument object from pyscpi or pyvisa
     """
 
     inst.write(f':AUToscale')
     inst.query('*OPC?')
 
 
-def setTimeAxis(inst: scpi.Instrument | visa.resources.Resource, scale: float, position: float) -> None:
+def setTimeAxis(inst, scale: float, position: float) -> None:
     """Sets the time axis of the oscilloscope.
 
-    :param inst: The instrument object
+    :param inst: The instrument object from pyscpi or pyvisa
     :param scale: The scale of the time axis in seconds
     :param position: The position of the time axis from the trigger in seconds
     """
@@ -195,10 +204,10 @@ def setTimeAxis(inst: scpi.Instrument | visa.resources.Resource, scale: float, p
     inst.query('*OPC?')
 
 
-def setChannelAxis(inst: scpi.Instrument | visa.resources.Resource, channel: int, scale: float, offset: float) -> None:
+def setChannelAxis(inst, channel: int, scale: float, offset: float) -> None:
     """Sets the channel axis (y-axis) of the oscilloscope.
 
-    :param inst: The instrument object
+    :param inst: The instrument object from pyscpi or pyvisa
     :param channel: The channel to set
     :param scale: The scale of the channel axis in volts
     :param offset: The offset of the channel axis in volts
@@ -208,10 +217,10 @@ def setChannelAxis(inst: scpi.Instrument | visa.resources.Resource, channel: int
     inst.query('*OPC?')
 
 
-def setWGenOutput(inst: scpi.Instrument | visa.resources.Resource, state: int | str) -> None:
+def setWGenOutput(inst, state: int | str) -> None:
     """Sets the output state of the waveform generator. (Only available on specific models)
 
-    :param inst: The instrument object
+    :param inst: The instrument object from pyscpi or pyvisa
     :param state: The state to set the output to (0 or 1) or ('OFF' or 'ON')
     """
 
@@ -219,10 +228,10 @@ def setWGenOutput(inst: scpi.Instrument | visa.resources.Resource, state: int |
     inst.query('*OPC?')
 
 
-def setWGenSin(inst: scpi.Instrument | visa.resources.Resource, amp: float, offset: float, freq: float) -> None:
+def setWGenSin(inst, amp: float, offset: float, freq: float) -> None:
     """Sets the waveform generator to a sine wave. (Only available on specific models)
 
-    :param inst: The instrument object
+    :param inst: The instrument object from pyscpi or pyvisa
     :param amp: The amplitude of the sine wave in volts
     :param offset: The offset of the sine wave in volts
     :param freq: The frequency of the sine wave in Hz. The frequency can be adjusted from 100 mHz to 20 MHz.
@@ -235,10 +244,10 @@ def setWGenSin(inst: scpi.Instrument | visa.resources.Resource, amp: float, offs
     inst.query('*OPC?')
 
 
-def setWGenSquare(inst: scpi.Instrument | visa.resources.Resource, v0: float, v1: float, offset: float, freq: float, dutyCycle: float) -> None:
+def setWGenSquare(inst, v0: float, v1: float, offset: float, freq: float, dutyCycle: int) -> None:
     """Sets the waveform generator to a square wave. (Only available on specific models)
 
-    :param inst: The instrument object
+    :param inst: The instrument object from pyscpi or pyvisa
     :param v0: The voltage of the low state in volts
     :param v1: The voltage of the high state in volts
     :param offset: The offset of the square wave in volts
@@ -255,10 +264,10 @@ def setWGenSquare(inst: scpi.Instrument | visa.resources.Resource, v0: float, v1
     inst.query('*OPC?')
 
 
-def setWGenRamp(inst: scpi.Instrument | visa.resources.Resource, v0: float, v1: float, offset: float, freq: float, symmetry: float) -> None:
+def setWGenRamp(inst, v0: float, v1: float, offset: float, freq: float, symmetry: float) -> None:
     """Sets the waveform generator to a ramp wave. (Only available on specific models)
 
-    :param inst: The instrument object
+    :param inst: The instrument object from pyscpi or pyvisa
     :param v0: The voltage of the low state in volts
     :param v1: The voltage of the high state in volts
     :param offset: The offset of the ramp wave in volts
@@ -275,10 +284,10 @@ def setWGenRamp(inst: scpi.Instrument | visa.resources.Resource, v0: float, v1:
     inst.query('*OPC?')
 
 
-def setWGenPulse(inst: scpi.Instrument | visa.resources.Resource, v0: float, v1: float, offset: float, period: float, pulseWidth: float) -> None:
+def setWGenPulse(inst, v0: float, v1: float, offset: float, period: float, pulseWidth: float) -> None:
     """Sets the waveform generator to a pulse wave. (Only available on specific models)
 
-    :param inst: The instrument object
+    :param inst: The instrument object from pyscpi or pyvisa
     :param v0: The voltage of the low state in volts
     :param v1: The voltage of the high state in volts
     :param offset: The offset of the pulse wave in volts
@@ -295,10 +304,10 @@ def setWGenPulse(inst: scpi.Instrument | visa.resources.Resource, v0: float, v1:
     inst.query('*OPC?')
 
 
-def setWGenDC(inst: scpi.Instrument | visa.resources.Resource, offset: float) -> None:
+def setWGenDC(inst, offset: float) -> None:
     """Sets the waveform generator to a DC wave. (Only available on specific models)
 
-    :param inst: The instrument object
+    :param inst: The instrument object from pyscpi or pyvisa
     :param offset: The offset of the DC wave in volts
     """
 
@@ -307,10 +316,10 @@ def setWGenDC(inst: scpi.Instrument | visa.resources.Resource, offset: float) ->
     inst.query('*OPC?')
 
 
-def setWGenNoise(inst: scpi.Instrument | visa.resources.Resource, v0: float, v1: float, offset: float) -> None:
+def setWGenNoise(inst, v0: float, v1: float, offset: float) -> None:
     """Sets the waveform generator to a noise wave. (Only available on specific models)
 
-    :param inst: The instrument object
+    :param inst: The instrument object from pyscpi or pyvisa
     :param v0: The voltage of the low state in volts
     :param v1: The voltage of the high state in volts
     :param offset: The offset of the noise wave in volts
@@ -321,3 +330,8 @@ def setWGenNoise(inst: scpi.Instrument | visa.resources.Resource, v0: float, v1:
     inst.write(f':WGEN:VOLTage:HIGH {v1}')
     inst.write(f':WGEN:VOLTage:OFFSet {offset}')
     inst.query('*OPC?')
+
+
+def _log(msg, EnableLog: bool) -> None:
+    if EnableLog:
+        print(msg)
